/*
 * test_taditerator.c
 *
 * Copyright (c) 2012 project bchan
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source
 *    distribution.
 *
 */

#include "test_tad.h"

#include "taditerator.h"

#include	<basic.h>
#include	<bstdio.h>
#include	<bstdlib.h>
#include	<bstring.h>

#include    <unittest_driver.h>

LOCAL UB test_taditerator_testdata01[] = {
	0xe0, 0xff, 0x06, 0x00, 0x00, 0x00, 0x02, 0x00,
	0x22, 0x01, 0xe1, 0xff, 0x18, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x88, 0xff,
	0x88, 0xff, 0x21, 0x00, 0x00, 0x00, 0xa0, 0xff,
	0x0e, 0x00, 0x00, 0x00, 0x7b, 0x05, 0xe0, 0x03,
	0x5e, 0x00, 0x46, 0x00, 0x6c, 0x00, 0x55, 0x00,
	0xa0, 0xff, 0x0a, 0x00, 0x00, 0x01, 0x5e, 0x00,
	0x76, 0x00, 0x6c, 0x00, 0x55, 0x00, 0xa0, 0xff,
	0x22, 0x00, 0x00, 0x03, 0xa0, 0xff, 0x02, 0x00,
	0x00, 0x08, 0xa1, 0xff, 0x04, 0x00, 0x00, 0x00,
	0x00, 0x80, 0xa1, 0xff, 0x02, 0x00, 0x01, 0x01,
	0x5d, 0x21, 0xad, 0xff, 0x04, 0x00, 0x00, 0x00,
	0xc8, 0x00, 0x5d, 0x21, 0xa0, 0xff, 0x04, 0x00,
	0x00, 0x04, 0x00, 0x80, 0xa4, 0xff, 0x24, 0x00,
	0x11, 0x08, 0x22, 0x21, 0x23, 0x21, 0x24, 0x21,
	0x25, 0x21, 0x2b, 0x21, 0x2c, 0x21, 0x47, 0x21,
	0x49, 0x21, 0x4b, 0x21, 0x4d, 0x21, 0x4f, 0x21,
	0x51, 0x21, 0x53, 0x21, 0x55, 0x21, 0x57, 0x21,
	0x59, 0x21, 0x5b, 0x21, 0xa4, 0xff, 0x18, 0x00,
	0x11, 0x09, 0x46, 0x21, 0x48, 0x21, 0x4a, 0x21,
	0x4c, 0x21, 0x4e, 0x21, 0x50, 0x21, 0x52, 0x21,
	0x54, 0x21, 0x56, 0x21, 0x58, 0x21, 0x5a, 0x21,
	0xa1, 0xff, 0x24, 0x00, 0x00, 0x02, 0x04, 0x01,
	0x04, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x0b, 0x00, 0x48, 0x00, 0x90, 0x00, 0xd8, 0x00,
	0x20, 0x01, 0x68, 0x01, 0xb0, 0x01, 0xf8, 0x01,
	0x40, 0x02, 0x88, 0x02, 0xd0, 0x02, 0x18, 0x03,
	0xa1, 0xff, 0x04, 0x00, 0x01, 0x00, 0x04, 0x01,
	0x46, 0x23, 0x52, 0x23, 0x4f, 0x23, 0x4d, 0x23,
	0x27, 0x21, 0x21, 0x21, 0x6e, 0x23, 0x61, 0x23,
	0x6d, 0x23, 0x65, 0x23, 0x0a, 0x00, 0x6d, 0x23,
	0x61, 0x23, 0x69, 0x23, 0x6c, 0x23, 0x27, 0x21,
	0x21, 0x21, 0x6d, 0x23, 0x61, 0x23, 0x69, 0x23,
	0x6c, 0x23, 0x0a, 0x00, 0x0a, 0x00, 0x62, 0x23,
	0x6f, 0x23, 0x64, 0x23, 0x79, 0x23, 0xe2, 0xff,
	0x00, 0x00,
};

LOCAL UNITTEST_RESULT test_taditerator_1()
{
	taditerator_t iter;
	TADITERATOR_RESULTTYPE result;
	W segsize;
	TC ch;
	UB *data;
	LTADSEG *seg;

	taditerator_initialize(&iter, (TC*)test_taditerator_testdata01, sizeof(test_taditerator_testdata01)/sizeof(TC));

	for (;;) {
		result = taditerator_next(&iter, NULL, &ch, &seg, &segsize, &data);
		if (result == TADITERATOR_RESULTTYPE_END) {
			break;
		}
	}

	taditerator_finalize(&iter);

	return UNITTEST_RESULT_PASS;
}

LOCAL UNITTEST_RESULT test_taditerator_2()
{
	taditerator_t iter;
	taditerator_result result;

	taditerator_initialize(&iter, (TC*)test_taditerator_testdata01, sizeof(test_taditerator_testdata01)/sizeof(TC));

	for (;;) {
		taditerator_next2(&iter, &result);
		if (result.type == TADITERATOR_RESULTTYPE_END) {
			break;
		}
	}

	taditerator_finalize(&iter);

	return UNITTEST_RESULT_PASS;
}

EXPORT VOID test_taditerator_main(unittest_driver_t *driver)
{
	UNITTEST_DRIVER_REGIST(driver, test_taditerator_1);
	UNITTEST_DRIVER_REGIST(driver, test_taditerator_2);
}
